﻿using Microsoft.Extensions.Logging;
using ProjectSellPC.Repos;
using System;
using System.Collections.Generic;
using System.Linq;

namespace ProjectSellPC.DocBuilder
{
    internal class TableReport
    {
        private readonly IChequeRepository _chequeRepository;
        private readonly IClientRepository _clientRepository;
        private readonly IProductInChequeRepository _productInChequeRepository;
        private readonly ILogger<TableReport> _logger;

        internal static readonly string[] Headers = { "Клиент", "Дата", "Количество" };

        public TableReport(IChequeRepository chequeRepository, IClientRepository clientRepository, IProductInChequeRepository productInChequeRepository, ILoggerFactory loggerFactory)
        {
            _chequeRepository = chequeRepository ?? throw new ArgumentNullException(nameof(chequeRepository));
            _clientRepository = clientRepository ?? throw new ArgumentNullException(nameof(clientRepository));
            _productInChequeRepository = productInChequeRepository ?? throw new ArgumentNullException(nameof(productInChequeRepository));
            _logger = loggerFactory.CreateLogger<TableReport>();
        }

        public bool CreateTable(string filePath, int productId, DateTime startDate, DateTime endDate)
        {
            try
            {
                new ExcelBuilder(filePath)
                    .AddHeader("Отчет по продажам товара", 0, 3)
                    .AddParagraph($"за период с {startDate.ToShortDateString()} по {endDate.ToShortDateString()}", 0)
                    .AddTable(new[] { 20, 20, 15 }, GetData(productId, startDate, endDate))
                    .Build();

                return true;
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при формировании отчета");
                return false;
            }
        }
        private List<string[]> GetData(int productId, DateTime startDate, DateTime endDate)
        {
            // Получение данных о товарах в чеках, содержащих указанный товар
            var productsInChecks = _productInChequeRepository.ReadAll()
                .Where(p => p.ProductID == productId) // Фильтр по товару
                .ToList();

            // Получение данных о чеках, содержащих указанный товар
            var checksWithProduct = _chequeRepository
                .ReadAll()
                .Where(c => c.PurchaseDate >= startDate && c.PurchaseDate <= endDate
                            && c.Products.Any(p => p.ProductID == productId))
                .Select(c => new
                {
                    CheckId = c.Id, // Номер чека
                    ClientId = c.Client?.Id ?? 0, // Получаем ClientId или 0, если клиент неизвестен
                    Date = c.PurchaseDate,
                    Quantity = c.Products
                        .Where(p => p.ProductID == productId)
                        .Sum(p => p.Count)
                })
                .OrderBy(x => x.Date)
                .ToList(); // Добавляем ToList() для выполнения запроса

            // Получение имен клиентов по ClientId
            var clientIds = checksWithProduct.Select(x => x.ClientId).Distinct().ToList();
            var clients = _clientRepository.ReadAll()
                .Where(c => clientIds.Contains(c.Id)) // Фильтруем клиентов по ClientId
                .ToDictionary(c => c.Id, c => c.Name); // Создаем словарь клиентов

            // Формирование итоговой таблицы
            return new List<string[]> { Headers }
                .Union(
                    checksWithProduct.Select(x => new string[]
                    {
                clients.ContainsKey(x.ClientId) ? clients[x.ClientId] : "Неизвестно", // Имя клиента
                x.Date.ToShortDateString(),
                x.Quantity.ToString()
                    })
                )
                .Union(new[]
                {
            new string[]
            {
                "Всего",
                "",
                checksWithProduct.Sum(x => x.Quantity).ToString()
            }
                })
                .ToList();
        }
    }
}