﻿using ProjectSellPC.Entites;
using ProjectSellPC.Reports;
using Dapper;
using Newtonsoft.Json;
using DocumentFormat.OpenXml.Drawing.Charts;
using Microsoft.Extensions.Logging;
using Npgsql;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Windows.Forms;

namespace ProjectSellPC.Repos.Impements
{
    public class ChequeRepo : IChequeRepository
    {
        private readonly IConnectionString _connectionString;
        private readonly ILogger<ChequeRepo> _logger;
        private readonly IClientRepository _clientRepository;
        private readonly IProductRepository _productRepository; // Добавляем зависимость

        public ChequeRepo(IConnectionString connectionString, ILoggerFactory loggerFactory,
                          IClientRepository clientRepository, IProductRepository productRepository)
        {
            _connectionString = connectionString ?? throw new ArgumentNullException(nameof(connectionString));
            _logger = loggerFactory.CreateLogger<ChequeRepo>();
            _clientRepository = clientRepository ?? throw new ArgumentNullException(nameof(clientRepository));
            _productRepository = productRepository ?? throw new ArgumentNullException(nameof(productRepository)); // Инициализация
        }
        private IDbConnection CreateConnection() => new NpgsqlConnection(_connectionString.ConnectionString);
        public void Create(Cheque cheque)
        {
            _logger.LogInformation("Создание чека");
            using (var connection = CreateConnection())
            {
                connection.Open();
                using (var transaction = connection.BeginTransaction())
                {
                    try
                    {
                        // Убедитесь, что ClientId установлен
                        if (cheque.Client == null || cheque.Client.Id == 0)
                        {
                            throw new InvalidOperationException("ClientId не может быть пустым или равным 0");
                        }

                        var chequeSql = "INSERT INTO \"cheque\" (\"clientid\", \"purchasedate\") VALUES (@ClientId, @PurchaseDate) RETURNING \"id\"";
                        cheque.Id = connection.ExecuteScalar<int>(chequeSql, new
                        {
                            ClientId = cheque.Client.Id,
                            PurchaseDate = cheque.PurchaseDate
                        }, transaction);

                        var productSql = "INSERT INTO \"productincheque\" (\"chequeid\", \"productid\", \"count\") VALUES (@ChequeId, @ProductID, @Count)";
                        foreach (var productInCheque in cheque.Products)
                        {
                            connection.Execute(productSql, new
                            {
                                ChequeId = cheque.Id,
                                ProductID = productInCheque.ProductID,
                                Count = productInCheque.Count
                            }, transaction);
                        }

                        transaction.Commit();
                    }
                    catch (Exception ex)
                    {
                        transaction.Rollback();
                        _logger.LogError(ex, "Ошибка при создании чека");
                        throw;
                    }
                }
            }
        }

        public Cheque Read(int id)
        {
            _logger.LogInformation("Чтение чека по ID: {id}", id);
            using (var connection = CreateConnection())
            {
                try
                {
                    var ChequeSql = "SELECT * FROM \"cheque\" WHERE \"id\" = @Id";
                    var Cheque = connection.QuerySingleOrDefault<Cheque>(ChequeSql, new { Id = id });

                    if (Cheque == null) return null;

                    Cheque.Client = _clientRepository.Read(Cheque.Client.Id);

                    var productSql = "SELECT * FROM \"productincheque\" WHERE \"chequeid\" = @ChequeId";
                    Cheque.Products = connection.Query<ProductInCheque>(productSql, new { ChequeId = Cheque.Id }).ToList();

                    return Cheque;
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "Ошибка при чтении чека");
                    throw;
                }
            }
        }
        public IEnumerable<Cheque> ReadAll(DateTime? startDate = null, DateTime? endDate = null, int? productId = null, int? clientId = null)
        {
            _logger.LogInformation("Чтение всех чеков с использованием фильтров");
            try
            {
                var builder = new QueryBuilder();

                // Добавляем условия фильтрации, если параметры заданы
                if (startDate.HasValue)
                    builder.AddCondition("\"cheque\".\"purchasedate\" >= @StartDate");
                if (endDate.HasValue)
                    builder.AddCondition("\"cheque\".\"purchasedate\" <= @EndDate");
                if (productId.HasValue)
                    builder.AddCondition("\"productincheque\".\"productid\" = @ProductId");
                if (clientId.HasValue)
                    builder.AddCondition("\"cheque\".\"clientid\" = @ClientId");

                // Формируем основной SQL-запрос
                var query = $"SELECT \"cheque\".*, \"client\".\"id\" AS \"clientid\", \"client\".\"name\" AS \"clientname\" " +
                            $"FROM \"cheque\" " +
                            $"LEFT JOIN \"client\" ON \"cheque\".\"clientid\" = \"client\".\"id\" " +
                            $"LEFT JOIN \"productincheque\" ON \"productincheque\".\"chequeid\" = \"cheque\".\"id\" " +
                            $"{builder.Build()} " +
                            $"ORDER BY \"cheque\".\"purchasedate\"";

                // Выполняем запрос
                using var connection = CreateConnection();
                var cheques = connection.Query<Cheque, int, string, Cheque>(
                    query,
                    (cheque, clientId, clientName) =>
                    {
                        cheque.Client = new Client { Id = clientId, Name = clientName ?? "Неизвестно" };
                        return cheque;
                    },
                    new { StartDate = startDate, EndDate = endDate, ProductId = productId, ClientId = clientId },
                    splitOn: "clientid,clientname"
                ).ToList();

                // Загружаем товары для каждого чека
                foreach (var cheque in cheques)
                {
                    var productQuery = "SELECT * FROM \"productincheque\" WHERE \"chequeid\" = @ChequeId";
                    cheque.Products = connection.Query<ProductInCheque>(productQuery, new { ChequeId = cheque.Id }).ToList();

                    // Загружаем названия товаров
                    foreach (var productInCheque in cheque.Products)
                    {
                        var product = _productRepository.Read(productInCheque.ProductID);
                        if (product != null)
                        {
                            productInCheque.ProductName = product.Name;
                        }
                        else
                        {
                            productInCheque.ProductName = "Неизвестный товар";
                        }
                    }
                }

                return cheques;
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при чтении всех чеков");
                throw;
            }
        }
    }
}

