﻿using ProjectSellPC.Entites;
using Dapper;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Npgsql;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;

namespace ProjectSellPC.Repos.Impements
{
    public class ProductsOnWarehouseRepo : IProductOnWarehouseRepository
    {
        private readonly IProductRepository _productRepository;
        private readonly IWarehouseRepository _warehouseRepository;
        private readonly IConnectionString _connectionString;
        private readonly ILogger<ProductsOnWarehouseRepo> _logger;
        public ProductsOnWarehouseRepo(IConnectionString connectionString, ILoggerFactory loggerFactory,
            IProductRepository productRepository,
            IWarehouseRepository warehouseRepository)
        {
            _connectionString = connectionString;
            _logger = loggerFactory.CreateLogger<ProductsOnWarehouseRepo>();
            _productRepository = productRepository ?? throw new ArgumentNullException(nameof(productRepository));
            _warehouseRepository = warehouseRepository ?? throw new ArgumentNullException(nameof(warehouseRepository));
        }

        private IDbConnection CreateConnection() => new NpgsqlConnection(_connectionString.ConnectionString);

        public void Create(ProductsOnWarehouse productsOnWarehouse)
        {
            _logger.LogInformation("Добавление продукта на склад");
            _logger.LogDebug("Продукт на складе: {json}", JsonConvert.SerializeObject(productsOnWarehouse));

            using (var connection = CreateConnection())
            {
                connection.Open();
                using (var transaction = connection.BeginTransaction())
                {
                    try
                    {
                        var sql = "INSERT INTO \"productsonwarehouse\" (\"productid\", \"warehouseid\", \"count\") " +
                                  "VALUES (@ProductId, @WarehouseId, @Count) RETURNING \"id\"";

                        productsOnWarehouse.Id = connection.ExecuteScalar<int>(sql, new
                        {
                            ProductId = productsOnWarehouse.Product.ID,
                            WarehouseId = productsOnWarehouse.Warehouse.Id,
                            Count = productsOnWarehouse.Count
                        }, transaction);

                        transaction.Commit();
                    }
                    catch (Exception ex)
                    {
                        transaction.Rollback();
                        _logger.LogError(ex, "Ошибка при добавлении продукта на склад");
                        throw;
                    }
                }
            }
        }

        public ProductsOnWarehouse Read(int id)
        {
            _logger.LogInformation("Получение продукта на складе по ID");
            _logger.LogDebug("ID продукта на складе: {id}", id);

            using (var connection = CreateConnection())
            {
                try
                {
                    var sql = "SELECT * FROM \"productsonwarehouse\" WHERE \"id\" = @Id";
                    var productOnWarehouse = connection.QuerySingleOrDefault<ProductsOnWarehouse>(sql, new { Id = id });
                    productOnWarehouse.Warehouse = _warehouseRepository.Read(productOnWarehouse.WarehouseId);
                    productOnWarehouse.Product = _productRepository.Read(productOnWarehouse.ProductId);

                    return productOnWarehouse;
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "Ошибка при получении продукта на складе");
                    throw;
                }
            }
        }

        public IEnumerable<ProductsOnWarehouse> ReadAll()
        {
            _logger.LogInformation("Получение всех продуктов на складе");

            using (var connection = CreateConnection())
            {
                try
                {
                    var sql = "SELECT * FROM \"productsonwarehouse\"";
                    var answer = connection.Query<ProductsOnWarehouse>(sql).ToList();
                    foreach (var productOnWarehouse in answer)
                    {
                        productOnWarehouse.Product = _productRepository.Read(productOnWarehouse.ProductId);
                        productOnWarehouse.Warehouse = _warehouseRepository.Read(productOnWarehouse.WarehouseId);
                    }
                    return answer;

                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "Ошибка при получении списка продуктов на складе");
                    throw;
                }
            }
        }

        public void Delete(int id)
        {
            _logger.LogInformation("Удаление товара со склада");
            _logger.LogDebug("Склад ID: {id}", id);

            try
            {
                using (var connection = CreateConnection())
                {
                    var sql = "DELETE FROM \"productsonwarehouse\" WHERE \"id\" = @Id";
                    connection.Execute(sql, new { Id = id });
                }
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при удалении");
                throw;
            }
        }
    }
}
